#include "config.h"
#include <debug.h>
#include <glib/gi18n-lib.h>

#include <fcntl.h>
#include <sys/stat.h>

#include <libnotify/notification.h>
#include <libnotify/notify.h>

#include <ircp_server.h>
#include <recvfile.h>
#include <savefiledlg.h>
#include <errno.h>
#include "resources.h"
#include "eggtrayicon.h"
#include "irda_misc.h"

enum recvfile_status {
	RCV_IDLE,
	RCV_LISTENING,
	RCV_RECEIVING
} RecvStatus;

static char RemoteName[256];

void ircp_info_cb(int event, char *param);

static ircp_server_t* IrcpServer = NULL;
static GIOChannel* IrcpServerChan = NULL;
static struct ProgressWindow* RecvFilePgswin = NULL;

static gboolean recvfile_onlisten(GIOChannel *source,
                                             GIOCondition condition,
                                             gpointer data);
static gboolean recvfile_onrecv(GIOChannel *source,
                                             GIOCondition condition,
                                             gpointer data);
void recvfile_launch_dialog(char* filename, char** localfilename, int* fd);

void log_file(const char*);

void recvfile_listen(char enabled)
{
	if(IrcpServer)
	{
		char* reason, *utf8, *logtext;
		if(IrcpServerChan) {
			g_source_remove_by_user_data(IrcpServer);
			g_io_channel_unref(IrcpServerChan);
			IrcpServerChan = NULL;
		}

		if(IrcpServer->finished)
			if(IrcpServer->success)
				reason = "succeeded";
			else
				reason = "failed";
		else
			reason = "aborted";
		
		if(IrcpServer->localfilename) {
			printf("filename %s\n", IrcpServer->localfilename);
			utf8 = filename_to_utf8(IrcpServer->localfilename);
			logtext = malloc(strlen(utf8)+30);
			sprintf(logtext, "receive %s %s", utf8, reason);
			log_file(logtext);
			free(logtext);
			free(utf8);
		}

		ircp_srv_close(IrcpServer);
		IrcpServer = NULL;
		RecvStatus = RCV_IDLE;
	}

	if(enabled) {
		int obexfd;

		IrcpServer = ircp_srv_open(ircp_info_cb);
		ircp_srv_listen(IrcpServer, ".");

		//create asynchronous i/o channel
		obexfd = OBEX_GetFD(IrcpServer->obexhandle);
		IrcpServerChan = g_io_channel_unix_new(obexfd);
		g_io_add_watch(IrcpServerChan, G_IO_IN, recvfile_onlisten, IrcpServer);
		RecvStatus = RCV_LISTENING;
	}
}

static void recvfile_abort()
{
	printf("Receiving abort\n");

	if(RecvFilePgswin) {
		progress_window_destroy(RecvFilePgswin);
		trayicon_animation_stop();
	}
	
	RecvFilePgswin = NULL;

	recvfile_listen(1);
}

static gboolean recvfile_onlisten(GIOChannel *source,
                                             GIOCondition condition,
                                             gpointer data)
{
	int ret, obexfd;
	struct _dev_info *devices;
    struct _dev_info *p;
	int numdev;
	
	ret = OBEX_HandleInput(IrcpServer->obexhandle, 1);
	
	//re-register callback
	g_source_remove_by_user_data(IrcpServer);
	g_io_channel_unref(IrcpServerChan);
	//create asynchronous i/o channel
	obexfd = OBEX_GetFD(IrcpServer->obexhandle);
	IrcpServerChan = g_io_channel_unix_new(obexfd);
	g_io_add_watch(IrcpServerChan, G_IO_IN, recvfile_onrecv, IrcpServer);
	
	RecvStatus = RCV_RECEIVING;

	devices = irda_get_hints_fd( &numdev, obexfd );
	if ( numdev <= 0 )
		printf("none remote device found\n");
	p = devices;
	while ( p ) {
		printf("name: %s, hint0 %x, hint1 %x\n",
				p->name, p->hints[0], p->hints[1]);
		strncpy(RemoteName, p->name, 255);
		RemoteName[255] = 0;
		p = p->next;
	}
	
	free_dev_info(devices);

	return FALSE;
}

static gboolean recvfile_onrecv(GIOChannel *source,
                                             GIOCondition condition,
                                             gpointer data)
{
	int ret;
	ircp_server_t* srv = IrcpServer;
	int obexfd;

	ret = OBEX_HandleInput(srv->obexhandle, 1);
	obexfd = OBEX_GetFD(IrcpServer->obexhandle);
	//printf("obexfd %d\n", obexfd);

	DEBUG(4, "HandleInput %d\n", ret);
	if (ret < 0) {
		srv->finished = TRUE;
		srv->success = FALSE;
	}

	if(RecvFilePgswin)
		progress_window_update(RecvFilePgswin, srv->finishedsize, srv->filelength);
	
		/*if(srv->filelength>0)
			gtk_progress_bar_set_fraction(
				GTK_PROGRESS_BAR(RecvFilePgswin->progressbar),
				(gdouble)srv->finishedsize / srv->filelength);
		else
			gtk_progress_bar_pulse(
				GTK_PROGRESS_BAR(RecvFilePgswin->progressbar));*/

	/*if(srv->filename && srv->fd < 0) {
		recvfile_launch_dialog(srv->filename, &srv->fd);
		if(srv->fd<0) {
			perror("No file open for writing, quit");
			srv->finished = TRUE;
			srv->success = FALSE;
		} else
			RecvFilePgswin = progress_window_new(srv->filename,
					NULL, NULL);
	}*/

	if(srv->fd >= 0 && !RecvFilePgswin) {
		char* utf8;
		utf8 = filename_to_utf8(srv->filename);
		RecvFilePgswin = progress_window_new(utf8,
					recvfile_abort, NULL);
		free(utf8);
		trayicon_animation_start();
	}

	/*if(srv->object) {
		recvfile_launch_dialog(&srv->fd);
		if(srv->fd<0) {
			perror("Target file open failed");
			OBEX_ObjectSetRsp(srv->object, 
					OBEX_RSP_FORBIDDEN,OBEX_RSP_FORBIDDEN);
			srv->finished = TRUE;
			srv->success = FALSE;
		} else
			OBEX_ObjectSetRsp(srv->object, OBEX_RSP_CONTINUE, OBEX_RSP_SUCCESS);

		OBEX_ObjectReadStream(srv->obexhandle,
				srv->object, NULL);
		srv->object = NULL;
	}*/
	
	if(srv->finished) {
		GtkWidget* dialog;
		GtkMessageType type;
		GtkWidget* parent = NULL;
		char* utf8;
		
		if (srv->localfilename)
		{
			//show transfer result dialog
			utf8 = filename_to_utf8(srv->localfilename);

			if (srv->success)
				type = GTK_MESSAGE_INFO;
			else
				type = GTK_MESSAGE_WARNING;
			
			if(RecvFilePgswin)
				parent = RecvFilePgswin->handle;
			
			dialog = gtk_message_dialog_new(parent,
											GTK_DIALOG_MODAL, type,
											GTK_BUTTONS_OK,
											"%s", utf8);
	
			if (srv->success) {
				gchar *title = NULL;
				gchar *msg = NULL;
				extern EggTrayIcon* TrayIcon;
				NotifyNotification *n = NULL;
				title = g_strdup (_("Infrared file received"));
				msg = g_strdup_printf (_("Saved to: %s\nSize: %d bytes"), utf8, srv->finishedsize);

				notify_init(PACKAGE);

				n = notify_notification_new (title,
                                       msg,
                                       WIN_ICON,
                                       GTK_WIDGET(TrayIcon));

				if (n != NULL) {
					notify_notification_set_urgency(n, NOTIFY_URGENCY_NORMAL);
					notify_notification_set_timeout (n, NOTIFY_EXPIRES_DEFAULT);

					if (!notify_notification_show (n, NULL))
						g_warning("failed to send notification\n");
					g_object_unref(G_OBJECT(n));
				}

			}
			else {
				gtk_message_dialog_format_secondary_text(dialog,
                                                         _("File receiving was aborted by remote"));
				gtk_dialog_run(GTK_DIALOG(dialog));
			}
			
			gtk_widget_destroy (dialog);
			free(utf8);
		}
		
		printf("srv finished, success %d\n", srv->success);
		if(srv->fd>=0) {
			printf("Close FD %d\n", srv->fd);
			close(srv->fd);
			srv->fd = -1;
		}
		if(RecvFilePgswin) {
			progress_window_destroy(RecvFilePgswin);
			trayicon_animation_stop();
		}
		RecvFilePgswin = NULL;

		recvfile_listen(1);

		return FALSE;
	} else {
		return TRUE;
	}
}

void recvfile_launch_dialog(char* filename_u, char** localfilename, int* fd)
{
	//GtkWidget *dialog;
	struct SaveFileDialog* dialog;

	*fd = -1;

	/*dialog = gtk_file_chooser_dialog_new ("Incoming file transfer save to",
				NULL,
				GTK_FILE_CHOOSER_ACTION_SAVE,
				GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
				GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
				NULL);*/
	dialog = savefile_dialog_new(_("Incoming File Transfer"), RemoteName);

	/*gtk_file_chooser_set_current_name(
		GTK_FILE_CHOOSER(dialog), filename);*/
	savefile_dialog_set_current_name(dialog, filename_u);
	//savefile_dialog_set_current_folder(dialog, "/home/xinzhen/Desktop");

	if (savefile_dialog_run (dialog) == GTK_RESPONSE_OK)
	{
		char *filename;

		//filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
		filename = savefile_dialog_get_filename(dialog);
		*fd = open(filename, O_WRONLY | O_CREAT | O_TRUNC, DEFFILEMODE);

		if(*fd<0) {
			char msgbuf[1024];
			char* unistr;
			GtkWidget* dialog;
	
			unistr = g_locale_to_utf8(strerror(errno), -1, NULL, NULL, NULL);
			snprintf(msgbuf, 1024, "%s, %s", _("Unable to open file for saving"), unistr);
			free(unistr);
			dialog = gtk_message_dialog_new(NULL, 0, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, msgbuf);
			gtk_dialog_run(GTK_DIALOG(dialog));
			gtk_widget_destroy(dialog);
		}
		
		*localfilename = filename;
	}else
		printf("recvfile dialog quit\n");

	savefile_dialog_destroy (dialog);
}
